library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity bin2bcd is
    port (
        clk, reset : in std_logic;
        bin_in : in std_logic_vector(7 downto 0);
        bcd_out : out std_logic_vector(11 downto 0)    
    );
end bin2bcd;

architecture Behavioral of bin2bcd is
    type State_t is (stIdle, stShift, stAdd);
    signal state_reg, next_state : State_t;
    
    signal shift_reg : std_logic_vector(19 downto 0);
    signal shift_cnt : integer range 0 to 7;
    signal bin_int : std_logic_vector(7 downto 0);
begin
    STATE_TRANSITION: process (clk) is
    begin
        if rising_edge(clk) then
            if reset = '1' then
                state_reg <= stIdle;
            else
                state_reg <= next_state;
            end if;
        end if;    
    end process STATE_TRANSITION;
    
    NEXT_STATE_LOGIC: process (state_reg, bin_in, bin_int, shift_cnt) is
    begin
        case state_reg is
            when stIdle =>
                if bin_in /= bin_int then
                    next_state <= stShift;
                else
                    next_state <= stIdle;
                end if;
            when stShift =>
                if shift_cnt = 0 then
                    next_state <= stIdle;
                else
                    next_state <= stAdd;
                end if;
            when stAdd =>
                next_state <= stShift;
            when others =>
                next_state <= stIdle;
        end case;    
    end process NEXT_STATE_LOGIC;
    
    SHIFT_PROC: process (clk) is
    begin
        if rising_edge(clk) then
            if state_reg = stIdle then
                if next_state = stShift then
                    shift_reg(19 downto 8) <= x"000";
                    shift_reg(7 downto 0) <= bin_in;
                end if;
            elsif state_reg = stShift then
                shift_reg(19 downto 1) <= shift_reg(18 downto 0);
                shift_reg(0) <= '0';
            else
                if unsigned(shift_reg(19 downto 16)) > 4 then
                    shift_reg(19 downto 16) <= 
                        std_logic_vector(unsigned(shift_reg(19 downto 16)) + 3);
                end if;
                if unsigned(shift_reg(15 downto 12)) > 4 then
                    shift_reg(15 downto 12) <= 
                        std_logic_vector(unsigned(shift_reg(15 downto 12)) + 3);
                end if;
                if unsigned(shift_reg(11 downto 8)) > 4 then
                    shift_reg(11 downto 8) <= 
                        std_logic_vector(unsigned(shift_reg(11 downto 8)) + 3);
                end if;            
            end if;        
        end if;    
    end process SHIFT_PROC;
    
    CNT_PROC: process (clk) is
    begin
        if rising_edge(clk) then
            if state_reg = stIdle then
                shift_cnt <= 7;
            elsif state_reg = stShift then
                shift_cnt <= shift_cnt - 1;
            end if;        
        end if; 
    end process CNT_PROC;
    
    REGS_PROC: process (clk) is
    begin
        if rising_edge(clk) then
            if reset = '1' then
                bcd_out <= x"000";
                bin_int <= x"00";
            elsif state_reg = stIdle then
                bcd_out <= shift_reg(19 downto 8);
                bin_int <= bin_in;
            end if;
        end if;    
    end process REGS_PROC;
end Behavioral;
